from __future__ import absolute_import

from plotly import exceptions, optional_imports
from plotly.figure_factory import utils
from plotly.graph_objs import graph_objs

# Optional imports, may be None for users that only use our core functionality.
np = optional_imports.get_module('numpy')


def validate_annotated_heatmap(z, x, y, annotation_text):
    """
    Annotated-heatmap-specific validations

    Check that if a text matrix is supplied, it has the same
    dimensions as the z matrix.

    See FigureFactory.create_annotated_heatmap() for params

    :raises: (PlotlyError) If z and text matrices do not  have the same
        dimensions.
    """
    if annotation_text is not None and isinstance(annotation_text, list):
        utils.validate_equal_length(z, annotation_text)
        for lst in range(len(z)):
            if len(z[lst]) != len(annotation_text[lst]):
                raise exceptions.PlotlyError("z and text should have the "
                                             "same dimensions")

    if x:
        if len(x) != len(z[0]):
            raise exceptions.PlotlyError("oops, the x list that you "
                                         "provided does not match the "
                                         "width of your z matrix ")

    if y:
        if len(y) != len(z):
            raise exceptions.PlotlyError("oops, the y list that you "
                                         "provided does not match the "
                                         "length of your z matrix ")


def create_annotated_heatmap(z, x=None, y=None, annotation_text=None,
                             colorscale='RdBu', font_colors=None,
                             showscale=False, reversescale=False,
                             **kwargs):
    """
    BETA function that creates annotated heatmaps

    This function adds annotations to each cell of the heatmap.

    :param (list[list]|ndarray) z: z matrix to create heatmap.
    :param (list) x: x axis labels.
    :param (list) y: y axis labels.
    :param (list[list]|ndarray) annotation_text: Text strings for
        annotations. Should have the same dimensions as the z matrix. If no
        text is added, the values of the z matrix are annotated. Default =
        z matrix values.
    :param (list|str) colorscale: heatmap colorscale.
    :param (list) font_colors: List of two color strings: [min_text_color,
        max_text_color] where min_text_color is applied to annotations for
        heatmap values < (max_value - min_value)/2. If font_colors is not
        defined, the colors are defined logically as black or white
        depending on the heatmap's colorscale.
    :param (bool) showscale: Display colorscale. Default = False
    :param kwargs: kwargs passed through plotly.graph_objs.Heatmap.
        These kwargs describe other attributes about the annotated Heatmap
        trace such as the colorscale. For more information on valid kwargs
        call help(plotly.graph_objs.Heatmap)

    Example 1: Simple annotated heatmap with default configuration
    ```
    import plotly.plotly as py
    import plotly.figure_factory as FF

    z = [[0.300000, 0.00000, 0.65, 0.300000],
         [1, 0.100005, 0.45, 0.4300],
         [0.300000, 0.00000, 0.65, 0.300000],
         [1, 0.100005, 0.45, 0.00000]]

    figure = FF.create_annotated_heatmap(z)
    py.iplot(figure)
    ```
    """

    # Avoiding mutables in the call signature
    font_colors = font_colors if font_colors is not None else []
    validate_annotated_heatmap(z, x, y, annotation_text)
    annotations = _AnnotatedHeatmap(z, x, y, annotation_text,
                                    colorscale, font_colors, reversescale,
                                    **kwargs).make_annotations()

    if x or y:
        trace = dict(type='heatmap', z=z, x=x, y=y, colorscale=colorscale,
                     showscale=showscale, **kwargs)
        layout = dict(annotations=annotations,
                      xaxis=dict(ticks='', dtick=1, side='top',
                                 gridcolor='rgb(0, 0, 0)'),
                      yaxis=dict(ticks='', dtick=1, ticksuffix='  '))
    else:
        trace = dict(type='heatmap', z=z, colorscale=colorscale,
                     showscale=showscale, **kwargs)
        layout = dict(annotations=annotations,
                      xaxis=dict(ticks='', side='top',
                                 gridcolor='rgb(0, 0, 0)',
                                 showticklabels=False),
                      yaxis=dict(ticks='', ticksuffix='  ',
                                 showticklabels=False))

    data = [trace]

    return graph_objs.Figure(data=data, layout=layout)


class _AnnotatedHeatmap(object):
    """
    Refer to TraceFactory.create_annotated_heatmap() for docstring
    """
    def __init__(self, z, x, y, annotation_text, colorscale,
                 font_colors, reversescale, **kwargs):

        self.z = z
        if x:
            self.x = x
        else:
            self.x = range(len(z[0]))
        if y:
            self.y = y
        else:
            self.y = range(len(z))
        if annotation_text is not None:
            self.annotation_text = annotation_text
        else:
            self.annotation_text = self.z
        self.colorscale = colorscale
        self.reversescale = reversescale
        self.font_colors = font_colors

    def get_text_color(self):
        """
        Get font color for annotations.

        The annotated heatmap can feature two text colors: min_text_color and
        max_text_color. The min_text_color is applied to annotations for
        heatmap values < (max_value - min_value)/2. The user can define these
        two colors. Otherwise the colors are defined logically as black or
        white depending on the heatmap's colorscale.

        :rtype (string, string) min_text_color, max_text_color: text
            color for annotations for heatmap values <
            (max_value - min_value)/2 and text color for annotations for
            heatmap values >= (max_value - min_value)/2
        """
        # Plotly colorscales ranging from a lighter shade to a darker shade
        colorscales = ['Greys', 'Greens', 'Blues',
                       'YIGnBu', 'YIOrRd', 'RdBu',
                       'Picnic', 'Jet', 'Hot', 'Blackbody',
                       'Earth', 'Electric', 'Viridis']
        # Plotly colorscales ranging from a darker shade to a lighter shade
        colorscales_reverse = ['Reds']
        if self.font_colors:
            min_text_color = self.font_colors[0]
            max_text_color = self.font_colors[-1]
        elif self.colorscale in colorscales and self.reversescale:
            min_text_color = '#000000'
            max_text_color = '#FFFFFF'
        elif self.colorscale in colorscales:
            min_text_color = '#FFFFFF'
            max_text_color = '#000000'
        elif self.colorscale in colorscales_reverse and self.reversescale:
            min_text_color = '#FFFFFF'
            max_text_color = '#000000'
        elif self.colorscale in colorscales_reverse:
            min_text_color = '#000000'
            max_text_color = '#FFFFFF'
        elif isinstance(self.colorscale, list):
            if 'rgb' in self.colorscale[0][1]:
                min_col = map(int,
                              self.colorscale[0][1].strip('rgb()').split(','))
                max_col = map(int,
                              self.colorscale[-1][1].strip('rgb()').split(','))
            elif '#' in self.colorscale[0][1]:
                    min_col = utils.hex_to_rgb(self.colorscale[0][1])
                    max_col = utils.hex_to_rgb(self.colorscale[-1][1])
            else:
                min_col = [255, 255, 255]
                max_col = [255, 255, 255]

            if (min_col[0]*0.299 + min_col[1]*0.587 + min_col[2]*0.114) > 186:
                min_text_color = '#000000'
            else:
                min_text_color = '#FFFFFF'
            if (max_col[0]*0.299 + max_col[1]*0.587 + max_col[2]*0.114) > 186:
                max_text_color = '#000000'
            else:
                max_text_color = '#FFFFFF'
        else:
            min_text_color = '#000000'
            max_text_color = '#000000'
        return min_text_color, max_text_color

    def get_z_mid(self):
        """
        Get the mid value of z matrix

        :rtype (float) z_avg: average val from z matrix
        """
        if np and isinstance(self.z, np.ndarray):
            z_min = np.amin(self.z)
            z_max = np.amax(self.z)
        else:
            z_min = min(min(self.z))
            z_max = max(max(self.z))
        z_mid = (z_max+z_min) / 2
        return z_mid

    def make_annotations(self):
        """
        Get annotations for each cell of the heatmap with graph_objs.Annotation

        :rtype (list[dict]) annotations: list of annotations for each cell of
            the heatmap
        """
        min_text_color, max_text_color = _AnnotatedHeatmap.get_text_color(self)
        z_mid = _AnnotatedHeatmap.get_z_mid(self)
        annotations = []
        for n, row in enumerate(self.z):
            for m, val in enumerate(row):
                font_color = min_text_color if val < z_mid else max_text_color
                annotations.append(
                    graph_objs.Annotation(
                        text=str(self.annotation_text[n][m]),
                        x=self.x[m],
                        y=self.y[n],
                        xref='x1',
                        yref='y1',
                        font=dict(color=font_color),
                        showarrow=False))
        return annotations



"""
graph_objs
==========

This package imports definitions for all of Plotly's graph objects. For more
information, run help(Obj) on any of the following objects defined here.

The reason for the package graph_objs and the module graph_objs is to provide
a clearer API for users.

"""
from __future__ import absolute_import

from plotly.graph_objs.graph_objs import *  # this is protected with __all__



import warnings
from .base import Renderer
from ..exporter import Exporter


class VincentRenderer(Renderer):
    def open_figure(self, fig, props):
        self.chart = None
        self.figwidth = int(props['figwidth'] * props['dpi'])
        self.figheight = int(props['figheight'] * props['dpi'])

    def draw_line(self, data, coordinates, style, label, mplobj=None):
        import vincent  # only import if VincentRenderer is used
        if coordinates != 'data':
            warnings.warn("Only data coordinates supported. Skipping this")
        linedata = {'x': data[:, 0],
                    'y': data[:, 1]}
        line = vincent.Line(linedata, iter_idx='x',
                            width=self.figwidth, height=self.figheight)

        # TODO: respect the other style settings
        line.scales['color'].range = [style['color']]

        if self.chart is None:
            self.chart = line
        else:
            warnings.warn("Multiple plot elements not yet supported")

    def draw_markers(self, data, coordinates, style, label, mplobj=None):
        import vincent  # only import if VincentRenderer is used
        if coordinates != 'data':
            warnings.warn("Only data coordinates supported. Skipping this")
        markerdata = {'x': data[:, 0],
                      'y': data[:, 1]}
        markers = vincent.Scatter(markerdata, iter_idx='x',
                                  width=self.figwidth, height=self.figheight)

        # TODO: respect the other style settings
        markers.scales['color'].range = [style['facecolor']]

        if self.chart is None:
            self.chart = markers
        else:
            warnings.warn("Multiple plot elements not yet supported")


def fig_to_vincent(fig):
    """Convert a matplotlib figure to a vincent object"""
    renderer = VincentRenderer()
    exporter = Exporter(renderer)
    exporter.run(fig)
    return renderer.chart



"""
dashboard_objs
==========

A module for creating and manipulating dashboard content. You can create
a Dashboard object, insert boxes, swap boxes, remove a box and get an HTML
preview of the Dashboard.
```
"""

import pprint

from plotly import exceptions, optional_imports
from plotly.utils import node_generator

IPython = optional_imports.get_module('IPython')

# default HTML parameters
MASTER_WIDTH = 400
MASTER_HEIGHT = 400
FONT_SIZE = 10

ID_NOT_VALID_MESSAGE = (
    "Your box_id must be a number in your dashboard. To view a "
    "representation of your dashboard run get_preview()."
)


def _empty_box():
    empty_box = {
        'type': 'box',
        'boxType': 'empty'
    }
    return empty_box


def _box(fileId='', shareKey=None, title=''):
    box = {
        'type': 'box',
        'boxType': 'plot',
        'fileId': fileId,
        'shareKey': shareKey,
        'title': title
    }
    return box


def _container(box_1=None, box_2=None, size=MASTER_HEIGHT,
               sizeUnit='px', direction='vertical'):
    if box_1 is None:
        box_1 = _empty_box()
    if box_2 is None:
        box_2 = _empty_box()

    container = {
        'type': 'split',
        'size': size,
        'sizeUnit': sizeUnit,
        'direction': direction,
        'first': box_1,
        'second': box_2
    }
    return container

dashboard_html = ("""
<!DOCTYPE HTML>
<html>
  <head>
    <style>
      body {{
        margin: 0px;
        padding: 0px;
      }}
    </style>
  </head>
  <body>
    <canvas id="myCanvas" width="400" height="400"></canvas>
    <script>
      var canvas = document.getElementById('myCanvas');
      var context = canvas.getContext('2d');
      <!-- Dashboard -->
      context.beginPath();
      context.rect(0, 0, {width}, {height});
      context.lineWidth = 2;
      context.strokeStyle = 'black';
      context.stroke();
      </script>
  </body>
</html>
""".format(width=MASTER_WIDTH, height=MASTER_HEIGHT))


def _draw_line_through_box(dashboard_html, top_left_x, top_left_y, box_w,
                           box_h, direction='vertical'):
    is_horizontal = (direction == 'horizontal')

    if is_horizontal:
        new_top_left_x = top_left_x + box_w / 2
        new_top_left_y = top_left_y
        new_box_w = 1
        new_box_h = box_h
    else:
        new_top_left_x = top_left_x
        new_top_left_y = top_left_y + box_h / 2
        new_box_w = box_w
        new_box_h = 1

    html_box = """<!-- Draw some lines in -->
          context.beginPath();
          context.rect({top_left_x}, {top_left_y}, {box_w}, {box_h});
          context.lineWidth = 1;
          context.strokeStyle = 'black';
          context.stroke();
    """.format(top_left_x=new_top_left_x, top_left_y=new_top_left_y,
               box_w=new_box_w, box_h=new_box_h)

    index_for_new_box = dashboard_html.find('</script>') - 1
    dashboard_html = (dashboard_html[:index_for_new_box] + html_box +
                      dashboard_html[index_for_new_box:])
    return dashboard_html


def _add_html_text(dashboard_html, text, top_left_x, top_left_y, box_w, box_h):
    html_text = """<!-- Insert box numbers -->
          context.font = '{font_size}pt Times New Roman';
          context.textAlign = 'center';
          context.fillText({text}, {top_left_x} + 0.5*{box_w}, {top_left_y} + 0.5*{box_h});
    """.format(text=text, top_left_x=top_left_x, top_left_y=top_left_y,
               box_w=box_w, box_h=box_h, font_size=FONT_SIZE)

    index_to_add_text = dashboard_html.find('</script>') - 1
    dashboard_html = (dashboard_html[:index_to_add_text] + html_text +
                      dashboard_html[index_to_add_text:])
    return dashboard_html


class Dashboard(dict):
    """
    Dashboard class for creating interactive dashboard objects.

    Dashboards are dicts that contain boxes which hold plot information.
    These boxes can be arranged in various ways. The most basic form of
    a box is:

    ```
    {
        'type': 'box',
        'boxType': 'plot'
    }
    ```

    where 'fileId' can be set to the 'username:#' of your plot. The other
    parameters a box takes are `shareKey` (default is None) and `title`
    (default is '').

    `.get_preview()` should be called quite regularly to get an HTML
    representation of the dashboard in which the boxes in the HTML
    are labelled with on-the-fly-generated numbers or box ids which
    change after each modification to the dashboard.

    `.get_box()` returns the box located in the dashboard by calling
    its box id as displayed via `.get_preview()`.

    Example: Create a simple Dashboard object
    ```
    import plotly.dashboard_objs as dashboard

    box_1 = {
        'type': 'box',
        'boxType': 'plot',
        'fileId': 'username:some#',
        'title': 'box 1'
    }

    box_2 = {
        'type': 'box',
        'boxType': 'plot',
        'fileId': 'username:some#',
        'title': 'box 2'
    }

    box_3 = {
        'type': 'box',
        'boxType': 'plot',
        'fileId': 'username:some#',
        'title': 'box 3'
    }

    my_dboard = dashboard.Dashboard()
    my_dboard.insert(box_1)
    # my_dboard.get_preview()
    my_dboard.insert(box_2, 'above', 1)
    # my_dboard.get_preview()
    my_dboard.insert(box_3, 'left', 2)
    # my_dboard.get_preview()
    my_dboard.swap(1, 2)
    # my_dboard.get_preview()
    my_dboard.remove(1)
    # my_dboard.get_preview()
    ```
    """
    def __init__(self, content=None):
        if content is None:
            content = {}

        if not content:
            self['layout'] = None
            self['version'] = 2
            self['settings'] = {}
        else:
            self['layout'] = content['layout']
            self['version'] = content['version']
            self['settings'] = content['settings']

        self._set_container_sizes()

    def _compute_box_ids(self):
        box_ids_to_path = {}
        all_nodes = list(node_generator(self['layout']))

        for node in all_nodes:
            if (node[1] != () and node[0]['type'] == 'box'
                    and node[0]['boxType'] != 'empty'):
                try:
                    max_id = max(box_ids_to_path.keys())
                except ValueError:
                    max_id = 0
                box_ids_to_path[max_id + 1] = node[1]

        return box_ids_to_path

    def _insert(self, box_or_container, path):
        if any(first_second not in ['first', 'second']
               for first_second in path):
            raise exceptions.PlotlyError(
                "Invalid path. Your 'path' list must only contain "
                "the strings 'first' and 'second'."
            )

        if 'first' in self['layout']:
            loc_in_dashboard = self['layout']
            for index, first_second in enumerate(path):
                if index != len(path) - 1:
                    loc_in_dashboard = loc_in_dashboard[first_second]
                else:
                    loc_in_dashboard[first_second] = box_or_container

        else:
            self['layout'] = box_or_container

    def _make_all_nodes_and_paths(self):
        all_nodes = list(node_generator(self['layout']))

        # remove path 'second' as it's always an empty box
        all_paths = []
        for node in all_nodes:
            all_paths.append(node[1])
        path_second = ('second',)
        if path_second in all_paths:
            all_paths.remove(path_second)
        return all_nodes, all_paths

    def _set_container_sizes(self):
        if self['layout'] is None:
            return

        all_nodes, all_paths = self._make_all_nodes_and_paths()

        # set dashboard_height proportional to max_path_len
        max_path_len = max(len(path) for path in all_paths)
        dashboard_height = 500 + 250 * max_path_len
        self['layout']['size'] = dashboard_height
        self['layout']['sizeUnit'] = 'px'

        for path in all_paths:
            if len(path) != 0:
                if self._path_to_box(path)['type'] == 'split':
                    self._path_to_box(path)['size'] = 50
                    self._path_to_box(path)['sizeUnit'] = '%'

    def _path_to_box(self, path):
        loc_in_dashboard = self['layout']
        for first_second in path:
            loc_in_dashboard = loc_in_dashboard[first_second]
        return loc_in_dashboard

    def get_box(self, box_id):
        """Returns box from box_id number."""
        box_ids_to_path = self._compute_box_ids()
        loc_in_dashboard = self['layout']

        if box_id not in box_ids_to_path.keys():
            raise exceptions.PlotlyError(ID_NOT_VALID_MESSAGE)
        for first_second in box_ids_to_path[box_id]:
            loc_in_dashboard = loc_in_dashboard[first_second]
        return loc_in_dashboard

    def get_preview(self):
        """
        Returns JSON or HTML respresentation of the dashboard.

        If IPython is not imported, returns a pretty print of the dashboard
        dict. Otherwise, returns an IPython.core.display.HTML display of the
        dashboard.

        The algorithm used to build the HTML preview involves going through
        the paths of the node generator of the dashboard. The paths of the
        dashboard are sequenced through from shorter to longer and whether
        it's a box or container that lies at the end of the path determines
        the action.

        If it's a container, draw a line in the figure to divide the current
        box into two and store the specs of the resulting two boxes. If the
        path points to a terminal box (often containing a plot), then draw
        the box id in the center of the box.

        It's important to note that these box ids are generated on-the-fly and
        they do not necessarily stay assigned to the boxes they were once
        assigned to.
        """
        if IPython is None:
            pprint.pprint(self)
            return

        elif self['layout'] is None:
            return IPython.display.HTML(dashboard_html)

        x = 0
        y = 0
        box_w = MASTER_WIDTH
        box_h = MASTER_HEIGHT
        html_figure = dashboard_html
        box_ids_to_path = self._compute_box_ids()
        # used to store info about box dimensions
        path_to_box_specs = {}
        first_box_specs = {
            'top_left_x': x,
            'top_left_y': y,
            'box_w': box_w,
            'box_h': box_h
        }
        # uses tuples to store paths as for hashable keys
        path_to_box_specs[('first',)] = first_box_specs

        # generate all paths
        all_nodes, all_paths = self._make_all_nodes_and_paths()

        max_path_len = max(len(path) for path in all_paths)
        for path_len in range(1, max_path_len + 1):
            for path in [path for path in all_paths if len(path) == path_len]:
                current_box_specs = path_to_box_specs[path]

                if self._path_to_box(path)['type'] == 'split':
                    html_figure = _draw_line_through_box(
                        html_figure,
                        current_box_specs['top_left_x'],
                        current_box_specs['top_left_y'],
                        current_box_specs['box_w'],
                        current_box_specs['box_h'],
                        direction=self._path_to_box(path)['direction']
                    )

                    # determine the specs for resulting two boxes from split
                    is_horizontal = (
                        self._path_to_box(path)['direction'] == 'horizontal'
                    )
                    x = current_box_specs['top_left_x']
                    y = current_box_specs['top_left_y']
                    box_w = current_box_specs['box_w']
                    box_h = current_box_specs['box_h']

                    if is_horizontal:
                        new_box_w = box_w / 2
                        new_box_h = box_h
                        new_top_left_x = x + box_w / 2
                        new_top_left_y = y

                    else:
                        new_box_w = box_w
                        new_box_h = box_h / 2
                        new_top_left_x = x
                        new_top_left_y = y + box_h / 2

                    box_1_specs = {
                        'top_left_x': x,
                        'top_left_y': y,
                        'box_w': new_box_w,
                        'box_h': new_box_h
                    }
                    box_2_specs = {
                        'top_left_x': new_top_left_x,
                        'top_left_y': new_top_left_y,
                        'box_w': new_box_w,
                        'box_h': new_box_h
                    }

                    path_to_box_specs[path + ('first',)] = box_1_specs
                    path_to_box_specs[path + ('second',)] = box_2_specs

                elif self._path_to_box(path)['type'] == 'box':
                    for box_id in box_ids_to_path:
                        if box_ids_to_path[box_id] == path:
                            number = box_id

                    html_figure = _add_html_text(
                        html_figure, number,
                        path_to_box_specs[path]['top_left_x'],
                        path_to_box_specs[path]['top_left_y'],
                        path_to_box_specs[path]['box_w'],
                        path_to_box_specs[path]['box_h'],
                    )

        # display HTML representation
        return IPython.display.HTML(html_figure)

    def insert(self, box, side='above', box_id=None):
        """
        Insert a box into your dashboard layout.

        :param (dict) box: the box you are inserting into the dashboard.
        :param (str) side: specifies where your new box is going to be placed
            relative to the given 'box_id'. Valid values are 'above', 'below',
            'left', and 'right'.
        :param (int) box_id: the box id which is used as the reference box for
            the insertion of the box.

        Example:
        ```
        import plotly.dashboard_objs as dashboard

        box_1 = {
            'type': 'box',
            'boxType': 'plot',
            'fileId': 'username:some#',
            'title': 'box 1'
        }

        my_dboard = dashboard.Dashboard()
        my_dboard.insert(box_1)
        my_dboard.insert(box_1, 'left', 1)
        my_dboard.insert(box_1, 'below', 2)
        my_dboard.insert(box_1, 'right', 3)
        my_dboard.insert(box_1, 'above', 4)

        my_dboard.get_preview()
        ```
        """
        box_ids_to_path = self._compute_box_ids()

        # doesn't need box_id or side specified for first box
        if self['layout'] is None:
            self['layout'] = _container(box, _empty_box())
        else:
            if box_id is None:
                raise exceptions.PlotlyError(
                    "Make sure the box_id is specfied if there is at least "
                    "one box in your dashboard."
                )
            if box_id not in box_ids_to_path:
                raise exceptions.PlotlyError(ID_NOT_VALID_MESSAGE)
            if side == 'above':
                old_box = self.get_box(box_id)
                self._insert(
                    _container(box, old_box, direction='vertical'),
                    box_ids_to_path[box_id]
                )
            elif side == 'below':
                old_box = self.get_box(box_id)
                self._insert(
                    _container(old_box, box, direction='vertical'),
                    box_ids_to_path[box_id]
                )
            elif side == 'left':
                old_box = self.get_box(box_id)
                self._insert(
                    _container(box, old_box, direction='horizontal'),
                    box_ids_to_path[box_id]
                )
            elif side == 'right':
                old_box = self.get_box(box_id)
                self._insert(
                    _container(old_box, box, direction='horizontal'),
                    box_ids_to_path[box_id]
                )
            else:
                raise exceptions.PlotlyError(
                    "If there is at least one box in your dashboard, you "
                    "must specify a valid side value. You must choose from "
                    "'above', 'below', 'left', and 'right'."
                )

        self._set_container_sizes()

    def remove(self, box_id):
        """
        Remove a box from the dashboard by its box_id.

        Example:
        ```
        import plotly.dashboard_objs as dashboard

        box_1 = {
            'type': 'box',
            'boxType': 'plot',
            'fileId': 'username:some#',
            'title': 'box 1'
        }

        my_dboard = dashboard.Dashboard()
        my_dboard.insert(box_1)
        my_dboard.remove(1)

        my_dboard.get_preview()
        ```
        """
        box_ids_to_path = self._compute_box_ids()
        if box_id not in box_ids_to_path:
            raise exceptions.PlotlyError(ID_NOT_VALID_MESSAGE)

        path = box_ids_to_path[box_id]
        if path != ('first',):
            container_for_box_id = self._path_to_box(path[:-1])
            if path[-1] == 'first':
                adjacent_path = 'second'
            elif path[-1] == 'second':
                adjacent_path = 'first'
            adjacent_box = container_for_box_id[adjacent_path]

            self._insert(adjacent_box, path[:-1])
        else:
            self['layout'] = None

        self._set_container_sizes()

    def swap(self, box_id_1, box_id_2):
        """
        Swap two boxes with their specified ids.

        Example:
        ```
        import plotly.dashboard_objs as dashboard

        box_1 = {
            'type': 'box',
            'boxType': 'plot',
            'fileId': 'username:first#',
            'title': 'first box'
        }

        box_2 = {
            'type': 'box',
            'boxType': 'plot',
            'fileId': 'username:second#',
            'title': 'second box'
        }

        my_dboard = dashboard.Dashboard()
        my_dboard.insert(box_1)
        my_dboard.insert(box_2, 'above', 1)

        # check box at box id 1
        box_at_1 = my_dboard.get_box(1)
        print(box_at_1)

        my_dboard.swap(1, 2)

        box_after_swap = my_dboard.get_box(1)
        print(box_after_swap)
        ```
        """
        box_ids_to_path = self._compute_box_ids()
        box_1 = self.get_box(box_id_1)
        box_2 = self.get_box(box_id_2)

        box_1_path = box_ids_to_path[box_id_1]
        box_2_path = box_ids_to_path[box_id_2]

        for pairs in [(box_1_path, box_2), (box_2_path, box_1)]:
            loc_in_dashboard = self['layout']
            for first_second in pairs[0][:-1]:
                loc_in_dashboard = loc_in_dashboard[first_second]
            loc_in_dashboard[pairs[0][-1]] = pairs[1]

        self._set_container_sizes()



"""
Matplotlib Exporter
===================
This submodule contains tools for crawling a matplotlib figure and exporting
relevant pieces to a renderer.
"""
import warnings
import io
from . import utils

import matplotlib
from matplotlib import transforms, collections


class Exporter(object):
    """Matplotlib Exporter

    Parameters
    ----------
    renderer : Renderer object
        The renderer object called by the exporter to create a figure
        visualization.  See mplexporter.Renderer for information on the
        methods which should be defined within the renderer.
    close_mpl : bool
        If True (default), close the matplotlib figure as it is rendered. This
        is useful for when the exporter is used within the notebook, or with
        an interactive matplotlib backend.
    """

    def __init__(self, renderer, close_mpl=True):
        self.close_mpl = close_mpl
        self.renderer = renderer

    def run(self, fig):
        """
        Run the exporter on the given figure

        Parmeters
        ---------
        fig : matplotlib.Figure instance
            The figure to export
        """
        # Calling savefig executes the draw() command, putting elements
        # in the correct place.
        fig.savefig(io.BytesIO(), format='png', dpi=fig.dpi)
        if self.close_mpl:
            import matplotlib.pyplot as plt
            plt.close(fig)
        self.crawl_fig(fig)

    @staticmethod
    def process_transform(transform, ax=None, data=None, return_trans=False,
                          force_trans=None):
        """Process the transform and convert data to figure or data coordinates

        Parameters
        ----------
        transform : matplotlib Transform object
            The transform applied to the data
        ax : matplotlib Axes object (optional)
            The axes the data is associated with
        data : ndarray (optional)
            The array of data to be transformed.
        return_trans : bool (optional)
            If true, return the final transform of the data
        force_trans : matplotlib.transform instance (optional)
            If supplied, first force the data to this transform

        Returns
        -------
        code : string
            Code is either "data", "axes", "figure", or "display", indicating
            the type of coordinates output.
        transform : matplotlib transform
            the transform used to map input data to output data.
            Returned only if return_trans is True
        new_data : ndarray
            Data transformed to match the given coordinate code.
            Returned only if data is specified
        """
        if isinstance(transform, transforms.BlendedGenericTransform):
            warnings.warn("Blended transforms not yet supported. "
                          "Zoom behavior may not work as expected.")

        if force_trans is not None:
            if data is not None:
                data = (transform - force_trans).transform(data)
            transform = force_trans

        code = "display"
        if ax is not None:
            for (c, trans) in [("data", ax.transData),
                               ("axes", ax.transAxes),
                               ("figure", ax.figure.transFigure),
                               ("display", transforms.IdentityTransform())]:
                if transform.contains_branch(trans):
                    code, transform = (c, transform - trans)
                    break

        if data is not None:
            if return_trans:
                return code, transform.transform(data), transform
            else:
                return code, transform.transform(data)
        else:
            if return_trans:
                return code, transform
            else:
                return code

    def crawl_fig(self, fig):
        """Crawl the figure and process all axes"""
        with self.renderer.draw_figure(fig=fig,
                                       props=utils.get_figure_properties(fig)):
            for ax in fig.axes:
                self.crawl_ax(ax)

    def crawl_ax(self, ax):
        """Crawl the axes and process all elements within"""
        with self.renderer.draw_axes(ax=ax,
                                     props=utils.get_axes_properties(ax)):
            for line in ax.lines:
                self.draw_line(ax, line)
            for text in ax.texts:
                self.draw_text(ax, text)
            for (text, ttp) in zip([ax.xaxis.label, ax.yaxis.label, ax.title],
                                   ["xlabel", "ylabel", "title"]):
                if(hasattr(text, 'get_text') and text.get_text()):
                    self.draw_text(ax, text, force_trans=ax.transAxes,
                                   text_type=ttp)
            for artist in ax.artists:
                # TODO: process other artists
                if isinstance(artist, matplotlib.text.Text):
                    self.draw_text(ax, artist)
            for patch in ax.patches:
                self.draw_patch(ax, patch)
            for collection in ax.collections:
                self.draw_collection(ax, collection)
            for image in ax.images:
                self.draw_image(ax, image)

            legend = ax.get_legend()
            if legend is not None:
                props = utils.get_legend_properties(ax, legend)
                with self.renderer.draw_legend(legend=legend, props=props):
                    if props['visible']:
                        self.crawl_legend(ax, legend)

    def crawl_legend(self, ax, legend):
        """
        Recursively look through objects in legend children
        """
        legendElements = list(utils.iter_all_children(legend._legend_box,
                                                      skipContainers=True))
        legendElements.append(legend.legendPatch)
        for child in legendElements:
            # force a large zorder so it appears on top
            child.set_zorder(1E6 + child.get_zorder())

            try:
                # What kind of object...
                if isinstance(child, matplotlib.patches.Patch):
                    self.draw_patch(ax, child, force_trans=ax.transAxes)
                elif isinstance(child, matplotlib.text.Text):
                    if (child is not legend.get_children()[-1]
                            and child.get_text() != 'None'):
                        self.draw_text(ax, child, force_trans=ax.transAxes)
                elif isinstance(child, matplotlib.lines.Line2D):
                    self.draw_line(ax, child, force_trans=ax.transAxes)
                elif isinstance(child, matplotlib.collections.Collection):
                    self.draw_collection(ax, child,
                                         force_pathtrans=ax.transAxes)
                else:
                    warnings.warn("Legend element %s not impemented" % child)
            except NotImplementedError:
                warnings.warn("Legend element %s not impemented" % child)

    def draw_line(self, ax, line, force_trans=None):
        """Process a matplotlib line and call renderer.draw_line"""
        coordinates, data = self.process_transform(line.get_transform(),
                                                   ax, line.get_xydata(),
                                                   force_trans=force_trans)
        linestyle = utils.get_line_style(line)
        if linestyle['dasharray'] in ['None', 'none', None]:
            linestyle = None
        markerstyle = utils.get_marker_style(line)
        if (markerstyle['marker'] in ['None', 'none', None]
                or markerstyle['markerpath'][0].size == 0):
            markerstyle = None
        label = line.get_label()
        if markerstyle or linestyle:
            self.renderer.draw_marked_line(data=data, coordinates=coordinates,
                                           linestyle=linestyle,
                                           markerstyle=markerstyle,
                                           label=label,
                                           mplobj=line)

    def draw_text(self, ax, text, force_trans=None, text_type=None):
        """Process a matplotlib text object and call renderer.draw_text"""
        content = text.get_text()
        if content:
            transform = text.get_transform()
            position = text.get_position()
            coords, position = self.process_transform(transform, ax,
                                                      position,
                                                      force_trans=force_trans)
            style = utils.get_text_style(text)
            self.renderer.draw_text(text=content, position=position,
                                    coordinates=coords,
                                    text_type=text_type,
                                    style=style, mplobj=text)

    def draw_patch(self, ax, patch, force_trans=None):
        """Process a matplotlib patch object and call renderer.draw_path"""
        vertices, pathcodes = utils.SVG_path(patch.get_path())
        transform = patch.get_transform()
        coordinates, vertices = self.process_transform(transform,
                                                       ax, vertices,
                                                       force_trans=force_trans)
        linestyle = utils.get_path_style(patch, fill=patch.get_fill())
        self.renderer.draw_path(data=vertices,
                                coordinates=coordinates,
                                pathcodes=pathcodes,
                                style=linestyle,
                                mplobj=patch)

    def draw_collection(self, ax, collection,
                        force_pathtrans=None,
                        force_offsettrans=None):
        """Process a matplotlib collection and call renderer.draw_collection"""
        (transform, transOffset,
         offsets, paths) = collection._prepare_points()

        offset_coords, offsets = self.process_transform(
            transOffset, ax, offsets, force_trans=force_offsettrans)
        path_coords = self.process_transform(
            transform, ax, force_trans=force_pathtrans)

        processed_paths = [utils.SVG_path(path) for path in paths]
        processed_paths = [(self.process_transform(
            transform, ax, path[0], force_trans=force_pathtrans)[1], path[1])
                           for path in processed_paths]

        path_transforms = collection.get_transforms()
        try:
            # matplotlib 1.3: path_transforms are transform objects.
            # Convert them to numpy arrays.
            path_transforms = [t.get_matrix() for t in path_transforms]
        except AttributeError:
            # matplotlib 1.4: path transforms are already numpy arrays.
            pass

        styles = {'linewidth': collection.get_linewidths(),
                  'facecolor': collection.get_facecolors(),
                  'edgecolor': collection.get_edgecolors(),
                  'alpha': collection._alpha,
                  'zorder': collection.get_zorder()}

        offset_dict = {"data": "before",
                       "screen": "after"}
        offset_order = offset_dict[collection.get_offset_position()]

        self.renderer.draw_path_collection(paths=processed_paths,
                                           path_coordinates=path_coords,
                                           path_transforms=path_transforms,
                                           offsets=offsets,
                                           offset_coordinates=offset_coords,
                                           offset_order=offset_order,
                                           styles=styles,
                                           mplobj=collection)

    def draw_image(self, ax, image):
        """Process a matplotlib image object and call renderer.draw_image"""
        self.renderer.draw_image(imdata=utils.image_to_base64(image),
                                 extent=image.get_extent(),
                                 coordinates="data",
                                 style={"alpha": image.get_alpha(),
                                        "zorder": image.get_zorder()},
                                 mplobj=image)
